""" Two api endpoints are currently planned

    * GET /tutor/:id to retrive information about some tutor
    * POST /tutor/:username/:email create a new tutor and email password
    * GET /tutorlist list of all tutors with their scores
"""
from django.contrib.auth import get_user_model
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import (APIClient, APIRequestFactory, APITestCase,
                                 force_authenticate)

from core.models import SubmissionSubscription, TutorSubmissionAssignment
from core.views import TutorApiViewSet
from util.factories import GradyUserFactory

NUMBER_OF_TUTORS = 3


class TutorDeleteTest(APITestCase):

    @classmethod
    def setUpTestData(cls):
        cls.factory = APIRequestFactory()
        cls.user_factory = GradyUserFactory()

    def setUp(self):
        self.tutor = self.user_factory.make_tutor(username='UFO')
        self.reviewer = self.user_factory.make_reviewer()
        self.request = self.factory.delete(reverse('tutor-detail',
                                                   args=['UFO']))
        self.view = TutorApiViewSet.as_view({'delete': 'destroy'})

        force_authenticate(self.request, user=self.reviewer)
        self.response = self.view(self.request, username='UFO')

    def test_can_delete_tutor_soapbox(self):
        """ see if the tutor was deleted """
        self.assertEqual(0, get_user_model().get_tutors().count())

    def test_user_is_deleted_too(self):
        """ see if the associated user was deleted (reviewer remains) """
        self.assertNotIn(self.tutor, get_user_model().objects.all())


class TutorListTests(APITestCase):

    @classmethod
    def setUpTestData(cls):
        cls.factory = APIRequestFactory()
        cls.user_factory = GradyUserFactory()

    def setUp(self):
        self.tutor_list = [self.user_factory.make_tutor()
                           for _ in range(NUMBER_OF_TUTORS)]
        self.reviewer = self.user_factory.make_reviewer()
        self.request = self.factory.get(reverse('tutor-list'))
        self.view = TutorApiViewSet.as_view({'get': 'list'})

        force_authenticate(self.request, user=self.reviewer)
        self.response = self.view(self.request)

    def test_can_access(self):
        self.assertEqual(self.response.status_code, status.HTTP_200_OK)

    def test_get_a_list_of_all_tutors(self):
        self.assertEqual(len(self.response.data), NUMBER_OF_TUTORS)

    def test_feedback_created_count_matches_database(self):
        def verify_fields(tutor_obj):
            t = get_user_model().objects.get(username=tutor_obj['username'])
            feedback_created_count = TutorSubmissionAssignment.objects.filter(
                is_done=True,
                subscription__feedback_stage=SubmissionSubscription.FEEDBACK_CREATION,  # noqa
                subscription__owner=t
            ).count()
            return feedback_created_count == tutor_obj['feedback_created']

        self.assertTrue(all(map(verify_fields, self.response.data)))

    def test_feedback_validated_count_matches_database(self):
        def verify_fields(tutor_obj):
            t = get_user_model().objects.get(username=tutor_obj['username'])
            feedback_validated_cnt = TutorSubmissionAssignment.objects.filter(
                is_done=True,
                subscription__feedback_stage=SubmissionSubscription.FEEDBACK_VALIDATION,  # noqa
                subscription__owner=t
            ).count()
            return feedback_validated_cnt == tutor_obj['feedback_created']

        self.assertTrue(all(map(verify_fields, self.response.data)))

    def test_sum_of_done_assignments(self):
        self.assertEqual(
            sum(obj['feedback_created'] + obj['feedback_validated']
                for obj in self.response.data),
            TutorSubmissionAssignment.objects.filter(is_done=True).count()
        )


class TutorCreateTests(APITestCase):

    USERNAME = 'some_weird_name'

    @classmethod
    def setUpTestData(cls):
        cls.factory = APIRequestFactory()
        cls.user_factory = GradyUserFactory()

    def setUp(self):
        self.reviewer = self.user_factory.make_reviewer()
        self.request = self.factory.post(reverse('tutor-list'),
                                         {'username': self.USERNAME})
        self.view = TutorApiViewSet.as_view({'post': 'create'})

        force_authenticate(self.request, user=self.reviewer)
        self.response = self.view(self.request, username=self.USERNAME)

    def test_can_access(self):
        self.assertEqual(self.response.status_code, status.HTTP_201_CREATED)

    def test_can_create(self):
        self.assertEqual(self.USERNAME,
                         get_user_model().get_tutors().first().username)


class TutorDetailViewTests(APITestCase):

    @classmethod
    def setUpTestData(cls):
        cls.user_factory = GradyUserFactory()

    def setUp(self):
        self.tutor = self.user_factory.make_tutor(username='fetterotto')
        self.reviewer = self.user_factory.make_reviewer()
        self.client = APIClient()
        self.client.force_authenticate(user=self.reviewer)

        url = reverse('tutor-detail', kwargs={'username': 'fetterotto'})
        self.response = self.client.get(url, format='json')

    def test_can_access(self):
        self.assertEqual(self.response.status_code, status.HTTP_200_OK)

    def test_can_view_tutor(self):
        self.assertEqual(self.response.data['username'],
                         self.tutor.username)
