import Vue from 'vue'
import Router from 'vue-router'
import Login from '@/pages/Login'
import StudentSubmissionPage from '@/pages/student/StudentSubmissionPage'
import StudentOverviewPage from '@/pages/reviewer/StudentOverviewPage'
import TutorOverviewPage from '@/pages/reviewer/TutorOverviewPage'
import SubscriptionWorkPage from '@/pages/SubscriptionWorkPage'
import SubscriptionEnded from '@/components/subscriptions/SubscriptionEnded'
import PageNotFound from '@/pages/PageNotFound'
import StartPageSelector from '@/pages/StartPageSelector'
import LayoutSelector from '@/pages/LayoutSelector'
import StudentSubmissionSideView from '@/pages/StudentSubmissionSideView'
import StudentListHelpCard from '@/components/student_list/StudentListHelpCard'
import FeedbackHistoryPage from '@/pages/base/FeedbackHistoryPage'
import FeedbackTable from '@/components/feedback_list/FeedbackTable'
import FeedbackListHelpCard from '@/components/feedback_list/FeedbackListHelpCard'
import VueInstance from '@/main'
import store from '@/store/store'

Vue.use(Router)

function denyAccess (next, redirect) {
  next(redirect.path)
  VueInstance.$notify({
    title: 'Access denied',
    text: "You don't have permission to view this.",
    type: 'error'
  })
}

function tutorOrReviewerOnly (to, from, next) {
  if (store.getters.isTutorOrReviewer) {
    next()
  } else {
    denyAccess(next, from.path)
  }
}

function reviewerOnly (to, from, next) {
  if (store.getters.isReviewer) {
    next()
  } else {
    denyAccess(next, from.path)
  }
}

function studentOnly (to, from, next) {
  if (store.getters.isStudent) {
    next()
  } else {
    next(false)
  }
}

function checkLoggedIn (to, from, next) {
  if (store.getters.isLoggedIn) {
    next()
  } else {
    next('/login/')
  }
}

const router = new Router({
  routes: [
    {
      path: '/login/',
      name: 'login',
      component: Login
    },
    {
      path: '',
      redirect: 'home',
      beforeEnter: checkLoggedIn,
      component: LayoutSelector,
      children: [
        {
          path: 'home',
          name: 'home',
          component: StartPageSelector
        },
        {
          path: 'subscription/ended',
          component: SubscriptionEnded
        },
        {
          path: 'subscription/:pk',
          name: 'subscription',
          beforeEnter: tutorOrReviewerOnly,
          component: SubscriptionWorkPage
        },
        {
          path: 'feedback',
          beforeEnter: tutorOrReviewerOnly,
          component: FeedbackHistoryPage,
          children: [
            {
              path: '',
              name: 'feedback',
              components: {
                left: FeedbackTable,
                right: FeedbackListHelpCard
              }
            },
            {
              path: ':submissionPk',
              components: {
                left: FeedbackTable,
                right: StudentSubmissionSideView
              },
              meta: {
                submissionSideView: true
              }
            },
            {
              path: ':id/detail',
              components: {
                left: FeedbackTable,
                right: PageNotFound
              }
            }
          ]
        },
        {
          path: 'student-overview',
          beforeEnter: reviewerOnly,
          component: StudentOverviewPage,
          children: [
            {
              path: '',
              name: 'student-overview',
              component: StudentListHelpCard
            },
            {
              path: 'student/:studentPk/submission/:submissionPk',
              name: 'submission-side-view',
              component: StudentSubmissionSideView,
              meta: {
                submissionSideView: true
              }
            }
          ]
        },
        {
          path: 'tutor-overview',
          name: 'tutor-overview',
          beforeEnter: reviewerOnly,
          component: TutorOverviewPage
        },
        {
          path: 'submission/:id',
          beforeEnter: studentOnly,
          component: StudentSubmissionPage
        }
      ]
    },
    {
      path: '*',
      name: 'page-not-found',
      component: PageNotFound
    }
  ]
})

export default router
