import {mut} from './mutations'
import {authMut} from '@/store/modules/authentication'
import {subNotesMut} from '@/store/modules/submission-notes'
import * as api from '@/api'
import router from '@/router/index'

function handleError (err, dispatch, fallbackMsg) {
  if (err.response) {
    if (err.response.status === 401) {
      dispatch('logout', "You've been logged out")
    } else {
      throw new Error(err.response.data)
    }
  } else {
    if (fallbackMsg) {
      throw new Error(fallbackMsg)
    }
  }
}

const actions = {
  async getSubscriptions ({commit, dispatch}) {
    try {
      const subscriptions = await api.fetchSubscriptions()
      commit(mut.SET_SUBSCRIPTIONS, subscriptions)
      return subscriptions
    } catch (err) {
      handleError(err, dispatch, 'Unable to fetch subscriptions')
    }
  },
  async getExamTypes ({commit, dispatch}) {
    try {
      const examTypes = await api.fetchExamType({})
      commit(mut.SET_EXAM_TYPES, examTypes)
    } catch (err) {
      handleError(err, dispatch, 'Unable to fetch exam mut')
    }
  },
  async subscribeTo ({commit, dispatch}, {type, key, stage}) {
    try {
      const subscription = await api.subscribeTo(type, key, stage)
      commit(mut.SET_SUBSCRIPTION, subscription)
      return subscription
    } catch (err) {
      handleError(err, dispatch, 'Subscribing unsuccessful')
    }
  },
  async deactivateSubscription ({commit, dispatch}, {subscription, pk}) {
    try {
      const subscriptionPk = subscription ? subscription.pk : pk
      await api.deactivateSubscription({pk: subscriptionPk})
      commit(mut.DELETE_SUBSCRIPTION, {pk: subscriptionPk})
    } catch (err) {
      handleError(err, dispatch, 'Unable to deactivate subscription')
    }
  },
  async updateSubmissionTypes ({commit, dispatch}, fields) {
    try {
      const submissionTypes = await api.fetchSubmissionTypes(fields)
      submissionTypes.forEach(type => {
        commit(mut.UPDATE_SUBMISSION_TYPE, type)
      })
    } catch (err) {
      handleError(err, dispatch, 'Unable to get submission types')
    }
  },
  async getAssignmentForSubscription ({commit, state, dispatch}, {subscription}) {
    if (subscription.assignments.length < 2) {
      try {
        const assignment = await api.createAssignment({subscription})
        commit(mut.ADD_ASSIGNMENT_TO_SUBSCRIPTION_QUEUE, {assignment})
        return assignment
      } catch (err) {
        handleError(err, dispatch, "Couldn't fetch assignment")
      }
    } else {
      return subscription.assignments[0]
    }
  },
  async deleteAssignment ({commit, state, dispatch}, {assignment}) {
    try {
      return await api.deleteAssignment({assignment})
    } catch (err) {
      handleError(err, dispatch, 'Unable to delete assignment')
    }
  },
  async getStudents ({commit, dispatch}, opt = {studentPks: [], fields: []}) {
    try {
      if (opt.studentPks.length === 0) {
        const students = await api.fetchAllStudents()
        commit(mut.SET_STUDENTS, students)
        return students
      } else {
        const students = await Promise.all(
          opt.studentPks.map(pk => api.fetchStudent({
            pk,
            fields: opt.fields
          }))
        )
        students.forEach(student => commit(mut.SET_STUDENT, student))
        return students
      }
    } catch (err) {
      console.log(err)
      handleError(err, dispatch, 'Unable to fetch student data')
    }
  },
  async getTutors ({commit, dispatch}) {
    try {
      const tutors = await api.fetchAllTutors()
      commit(mut.SET_TUTORS, tutors)
    } catch (err) {
      handleError(err, dispatch, 'Unable to fetch tutor data')
    }
  },
  async getAllFeedback ({commit, dispatch}, fields = []) {
    try {
      const feedback = await api.fetchAllFeedback({fields})
      commit(mut.SET_ALL_FEEDBACK, feedback)
      commit(mut.MAP_FEEDBACK_OF_SUBMISSION_TYPE)
    } catch (err) {
      handleError(err, dispatch, 'Unable to fetch feedback history')
    }
  },
  async getFeedback ({commit, dispatch}, {ofSubmission}) {
    try {
      const feedback = await api.fetchFeedback({ofSubmission})
      commit(mut.SET_FEEDBACK, feedback)
      return feedback
    } catch (err) {
      handleError(err, dispatch, `Unable to fetch feedback ${ofSubmission}`)
    }
  },
  async getSubmissionFeedbackTest ({commit, dispatch}, {pk}) {
    try {
      const submission = await api.fetchSubmissionFeedbackTests({pk})
      commit(mut.SET_SUBMISSION, submission)
    } catch (err) {
      handleError(err, dispatch, 'Unable to fetch submission')
    }
  },
  async getStatistics ({commit, dispatch}, opt) {
    try {
      const statistics = await api.fetchStatistics(opt)
      commit(mut.SET_STATISTICS, statistics)
    } catch (err) {
      handleError(err, dispatch, 'Unable to fetch statistics')
    }
  },
  async deleteComment ({commit, dispatch}, comment) {
    try {
      await api.deleteComment(comment)
    } catch (err) {
      handleError(err, dispatch, `Unable to delete comment: ${comment.text}`)
    }
  },
  async patchComment ({commit, dispatch}, comment) {
    try {
      return await api.patchComment(comment)
    } catch (err) {
      handleError(err, dispatch, `Unable to patch comment: ${comment.text}`)
    }
  },
  logout ({ commit }, message = '') {
    commit(mut.RESET_STATE)
    commit('submissionNotes/' + subNotesMut.RESET_STATE)
    commit(authMut.SET_MESSAGE, message)
    router.push({name: 'login'})
  }
}

export default actions
