#!/usr/local/bin/python3
""" a simple script that converts ilias exam output to readable json

The json output will look like this:
{
    "max.mustermann": { <<--- OR all uppercase letter of the name + username/matrikel_no  # noqa: E501
        "matrikel_no": "12345678",
        "name": "Mustermann, Max",
        "task_list": {
            "[task_id_1]": "print Hello World!",
            ....,
            "[task_id_n]": "#include <stdio.h> etc."
        }
    },
    ... ans so on
}

usage: convert.py [-h] [-u USERNAMES] [-n NUMBER_OF_TASKS] INFILE OUTFILE

positional arguments:
  INFILE                Ilias exam data
  OUTFILE               Where to write the final file

optional arguments:
  -h, --help            show this help message and exit
  -u USERNAMES, --usernames USERNAMES
                        a json dict matno -> email
  -n NUMBER_OF_TASKS, --NUMBER_OF_TASKS NUMBER_OF_TASKS
                        Where to write the final file


Author: Jan Maximilian Michal
Date: 30 March 2017
"""

import argparse
import json
import os
import re
import urllib.parse
from collections import defaultdict, namedtuple

from xlrd import open_workbook

parser = argparse.ArgumentParser()
parser.add_argument('INFILE', help='Ilias exam data')
parser.add_argument('OUTFILE', help='Where to write the final file')
parser.add_argument('-u', '--usernames', help='a json dict matno -> email')
parser.add_argument(
    '-n', '--NUMBER_OF_TASKS',
    default=0,  # don't check
    metavar='NUMBER_OF_TASKS',
    type=int,
    help='Where to write the final file')


# one user has one submission (code) per task
# yes, I know it is possible to name match groups via (?P<name>) but
# I like this solution better since it gets the job done nicely
user_t = namedtuple('user_head', 'name matrikel_no')

# one task has a title and id and hpfly code
task_head_re = re.compile(r'^Quellcode Frage (?P<title>.*?) ?(\d{8})?$')

# nor parsing the weird mat no
matno_re = re.compile(r'^(?P<matrikel_no>\d{8})-(\d+)-(\d+)$')


def converter(infile, usernames=None, number_of_tasks=0,):

    # Modify these iterators in order to change extraction behaviour

    def sheet_iter_meta(sheet):
        """ yield first and second col entry as tuple of (name, matnr) """
        for row in (sheet.row(i) for i in range(1, sheet.nrows)):
            match = re.search(matno_re, row[1].value)
            if match:
                yield row[0].value, match.group('matrikel_no')

    def sheet_iter_data(sheet):
        """ yields all source code titel and code tuples """
        def row(i):
            return sheet.row(i)
        for top, low in ((row(i), row(i + 1)) for i in range(sheet.nrows - 1)):
            if any(map(lambda c: c.ctype, top)) and 'Quell' in top[0].value:
                yield (' '.join(c.value for c in top),
                       ' '.join(c.value for c in low))

    # meta sheet contains ilias names usernames etc - data contains code
    meta, *data = open_workbook(infile, open(os.devnull, 'w')).sheets()

    # nice!
    name2mat = dict(sheet_iter_meta(meta))
    assert len(name2mat) == len(data), f'{len(name2mat)} names != {len(data)} sheets'  # noqa

    # from xls to lists and namedtuples
    # [ [user0, task0_h, code0, ..., taskn, coden ], ..., [...] ]
    root = []
    for user, sheet in zip(sheet_iter_meta(meta), data):
        root.append([user_t(*user)])
        for task, code in sheet_iter_data(sheet):
            task = re.search(task_head_re, task)
            root[-1].append(task.group('title'))
            root[-1].append(urllib.parse.unquote(code).strip())

    if number_of_tasks:
        for (user, *task_list) in sorted(root, key=lambda u: u[0].name):
            assert len(task_list) == number_of_tasks * 2

    mat_to_email = defaultdict(str)
    if usernames:
        with open(usernames) as data:
            mat_to_email.update(json.JSONDecoder().decode(data.read()))

    def get_username(user):
        if name2mat[user.name] in mat_to_email:
            return mat_to_email[name2mat[user.name]].split('@')[0]
        return ''.join(filter(str.isupper, user.name)) + name2mat[user.name]

    usernames = {user.name: get_username(user) for (user, *_) in root}

    # form list to json_like via comprehension
    # the format {userinitials + matrikel_no : {name:, matrikel_no:, tasklist:
    # {id:, ..., id:}}}
    return {
        usernames[user.name]: {
            'fullname': user.name,
            'email': mat_to_email[name2mat[user.name]],
            'matrikel_no': name2mat[user.name],
            'submissions': [
                {
                    "type": task,
                    "code": code,
                    "tests": {},
                } for task, code in zip(task_list[::2], task_list[1::2])
            ]
        } for (user, *task_list) in sorted(root, key=lambda u: u[0].name)
    }


def write_to_file(json_dict, outfile):
    # just encode python style
    with open(outfile, "w") as out:
        json.dump(json_dict, out, indent=2)

    print(f"Wrote data to {outfile}. Done.")


def main():
    args = parser.parse_args()
    json_dict = converter(args.INFILE, args.usernames, args.NUMBER_OF_TASKS)
    write_to_file(json_dict, args.OUTFILE)


if __name__ == '__main__':
    SCRIPT = True
    main()
